<?php
/**
 * Leafpub: Simple, beautiful publishing. (https://leafpub.org)
 *
 * @link      https://github.com/Leafpub/leafpub
 * @copyright Copyright (c) 2016 Leafpub Team
 * @license   https://github.com/Leafpub/leafpub/blob/master/LICENSE.md (GPL License)
 */

namespace Leafpub\Importer;

use Leafpub\Post,
    Leafpub\Leafpub,
    Leafpub\Tag,
    Leafpub\Upload,
    Leafpub\User,
    Leafpub\Database;

/**
* AbstractImporter
*
* methods for importing data from other blog platforms
* @package Leafpub\Importer
*
**/  
abstract class AbstractImporter {
    /**
    * Properties
    **/
    protected $_user, $_tags, $_categories, $_content, $_media, $_post_tags; // arrays
    protected $_loadMediaFilesRemote = false; // boolean
    protected $_loadMediaFilesLocal = false;
    protected $_loadUser = false;
    protected $_truncateTables = false;
    protected $_useCategoryAsTag = false;
    protected $_tmpPath = '';
    protected $_fileToParse; // String
    protected $_oldBlogUrl; // String - this adress will be deleted in filterPosts() in img src tags
    
    public function __construct($file){
        $this->_fileToParse = $file;
    }
    
    public function setOptions($options){
        $this->_loadMediaFilesRemote =  ($options['media'] == 'remote' ? true : false);
        $this->_loadMediaFilesLocal = ($options['media'] == 'local' ? true : false);
        $this->_loadUser = ($options['user'] == 'false' ? false : true);
        $this->_truncateTables = ($options['flush'] == 'false' ? false : true);
        $this->_useCategoryAsTag = ($options['category'] == 'false' ? false :true);
    }
    /**
    * This function has to be defined in the ImporterClass
    *
    * @return array
    *
    **/
    abstract public function parseFile();
    
    /**
    * Loads an external media file via CURL
    *
    * @param String $filename
    * @param String $url
    * @return void
    *
    **/
    protected function loadMediaFile($filename, $url){
        //Code to download the file
        
        // Where should we save the media files?
        // Every file in todays folder (year/month/day)?
        $mediaFile = fopen ($this->_tmpPath . '/' . $filename, 'w+');
        $handle = curl_init(str_replace(" ","%20",$url));
        curl_setopt($ch, CURLOPT_TIMEOUT, 50);
        
        curl_setopt($handle, CURLOPT_FILE, $mediaFile); 
        curl_setopt($handle, CURLOPT_FOLLOWLOCATION, true);
        
        curl_exec($handle); 
        curl_close($handle);
        fclose($mediaFile);
    }
    
    // In this abstract class this function filters all img tags and sets the correct img src path
    // In the ImporterClasses this function could be extended to strip Wordpress shortcode tags
    protected function filterContent($content){
        return str_replace($this->_oldBlogUrl, '/', $content);
    }
    
    // parseFile fills our protected arrays with data.
    // We are now saving the array data to DB; 
    public function importData(){
        Database::beginTransaction();
        if ($this->_truncateTables){
            Database::truncate('__post_tags');
            Database::truncate('__posts');
            Database::truncate('__tags');
        }
        if ($this->_loadUser){
            $this->_importUser();
        }
        $this->_importTags();
        $this->_importPosts();
        if ($this->_loadMediaFilesRemote || $this->_loadMediaFilesLocal){
            $this->_importMedia();
        }
        Database::commit();
        return true;
    }

    private function _importUser(){
        foreach($this->_user as $slug => $ds){
            try {
                User::add($slug, $ds);
            } catch (\Exception $e){
                //echo($e->getMessage());
            }
        }
        return true;
    }

    private function _importTags(){
        $data = array();
        if ($this->_useCategoryAsTag){
            $data = $this->_categories;
        } else {
            $data = $this->_tags;
        }
        foreach($data as $slug => $ds){
            try {
                Tag::add($slug, $ds);
            } catch (\Exception $e){
                //echo($e->getMessage());
            }
        }
        return true;
    }

    private function _importPosts(){
        foreach($this->_posts as $slug => $ds){
            try {
                Post::add($slug, $ds);
            } catch (\Exception $e){
                //echo($e->getMessage());
            }
        }
        return true;
    }
    /*
    private function _importPostTags(){

    }
    */
    private function _importMedia(){
        $info = array();
        if ($this->_loadMediaFilesRemote){
            // if we load media from an external server...
            $this->_tmpPath = Leafpub::path('content/uploads/import/' . uniqid());
            if (!Leafpub::makeDir($this->_tmpPath)){
                throw new \Exception('Couldn\'t create ' . $this->_tmpPath );
            }
        } elseif ($this->_loadMediaFilesLocal){
            // if we load from a temp dir of our server...
            $this->_tmpPath = Leafpub::path('content/uploads/import');
        } else {
            throw new \Exception('Haha, nice try....');
        }
        

        foreach ($this->_media as $file){
            $path = '';
            $filename = $file['filename'] . '.' . $file['extension'];
            // CURL load
            if ($this->_loadMediaFilesRemote){
                $this->loadMediaFile($filename, $file['url']);
                $path = $this->_tmpPath . '/' . $filename;
            }
            // Just copy the file
            if ($this->_loadMediaFilesLocal){
                $path = $this->_tmpPath . '/' . $file['attachedFile'];
            }
            if (is_file($path)){
                try {
                    Upload::add($filename, file_get_contents($path), $info);
                }
                catch (\Exception $e){

                }
            }
        }
        if ($this->_loadMediaFilesRemote){
            Leafpub::removeDir($this->_tmpPath);
        }
        return true;
    }
}
?>